@@ -3,13 +3,11 @@ from abc import ABCMeta, abstractmethod
 from contextlib import asynccontextmanager
 from dataclasses import dataclass
 from pathlib import Path
-from typing import AsyncIterator, Callable, Iterable, Literal, Optional, Union, cast
+from typing import Any, AsyncIterator, Callable, Iterable, Optional, cast
 
 from trio import run_process
 
-from .utils import TERMINAL_LOCK, T, async_cached
-
-LinkMethod = Literal["binary_wrapper"]
+from .utils import TERMINAL_LOCK, RunArg, Some, T, async_cached, intercalate, unsome
 
 
 class OS(metaclass=ABCMeta):
@@ -70,7 +68,7 @@ class Driver(metaclass=ABCMeta):
     @abstractmethod
     async def run_(
         self,
-        *args: Union[str, Path],
+        *args: RunArg,
         check: bool = True,
         input: Optional[bytes] = None,  # pylint:disable=redefined-builtin
         capture_output: bool = False,
@@ -80,17 +78,17 @@ class Driver(metaclass=ABCMeta):
 
     async def run(
         self,
-        *args: Union[str, Path],
+        *args: RunArg,
         input: Optional[bytes] = None,  # pylint:disable=redefined-builtin
         silent: bool = False,
     ) -> None:
         await self.run_(*args, input=input, silent=silent)
 
-    async def run_ok(self, *args: Union[str, Path]) -> bool:
+    async def run_ok(self, *args: RunArg) -> bool:
         result = await self.run_(*args, check=False)
         return result.ok
 
-    async def run_output(self, *args: Union[str, Path], silent: bool = False) -> str:
+    async def run_output(self, *args: RunArg, silent: bool = False) -> str:
         result = await self.run_(*args, capture_output=True, silent=silent)
         return cast(str, result.output)
 
@@ -117,6 +115,7 @@ class Driver(metaclass=ABCMeta):
             return "root"
         return await self.run_output("whoami")
 
+    @async_cached
     async def home(self) -> Path:
         path = "~root" if self.root else "~"
         result = await self.with_root(False).run_output("sh", "-c", f"eval echo {path}")
@@ -125,6 +124,39 @@ class Driver(metaclass=ABCMeta):
     async def local(self) -> Path:
         return await self.home() / ".local"
 
+    async def find(
+        self,
+        path: Path,
+        *,
+        name: Some[str] = None,
+        file_type: Some[str] = None,
+        mindepth: Optional[int] = None,
+        maxdepth: Optional[int] = None,
+    ) -> list[Path]:
+        args: list[RunArg] = ["find", path]
+
+        def add_some_arg(arg: str, values: list[str]) -> None:
+            if values:
+                if len(values) > 1:
+                    args.append("(")
+                args.extend(intercalate("-o", ([arg, v] for v in values)))
+                if len(values) > 1:
+                    args.append(")")
+
+        def add_optional_arg(arg: str, value: Optional[Any]) -> None:
+            if value is not None:
+                args.extend([arg, str(value)])
+
+        add_optional_arg("-mindepth", mindepth)
+        add_optional_arg("-maxdepth", maxdepth)
+        add_some_arg("-name", unsome(name))
+        add_some_arg("-type", unsome(file_type))
+
+        args.append("-print0")
+
+        output = await self.run_output(*args)
+        return [Path(result) for result in output.split("\0") if result]
+
     @asynccontextmanager
     async def tempfile(self) -> AsyncIterator[Path]:
         path = Path(await self.run_output("mktemp"))
@@ -146,22 +178,18 @@ class Driver(metaclass=ABCMeta):
         return await self.run_output("cat", path)
 
     async def write_file(self, path: Path, content: str) -> None:
+        await self.makedirs(path.parent)
+        await self.rm(path)
         await self.run("cp", "/dev/stdin", path, input=content.encode())
 
     async def link(
         self,
         source: Path,
         target: Path,
-        *,
-        method: Optional[LinkMethod] = None,
     ) -> None:
         await self.makedirs(target.parent)
         await self.rm(target)
-        if method == "binary_wrapper":
-            await self.write_file(target, f'#!/bin/sh\nexec "{source}" "$@"')
-            await self.make_executable(target)
-        else:
-            await self.run("ln", "-s", "-f", source, target)
+        await self.run("ln", "-s", "-f", source, target)
 
     @async_cached
     async def os(self) -> OS:
@@ -177,14 +205,12 @@ class Driver(metaclass=ABCMeta):
 
 
 class SubprocessDriver(Driver, metaclass=ABCMeta):
-    def prepare_command(
-        self, args: Iterable[Union[str, Path]]
-    ) -> list[Union[str, Path]]:
+    def prepare_command(self, args: Iterable[RunArg]) -> list[RunArg]:
         return list(args)
 
     async def run_(
         self,
-        *args: Union[str, Path],
+        *args: RunArg,
         check: bool = True,
         input: Optional[bytes] = None,  # pylint:disable=redefined-builtin
         capture_output: bool = False,
@@ -210,9 +236,7 @@ class SubprocessDriver(Driver, metaclass=ABCMeta):
 
 
 class LocalDriver(SubprocessDriver):
-    def prepare_command(
-        self, args: Iterable[Union[str, Path]]
-    ) -> list[Union[str, Path]]:
+    def prepare_command(self, args: Iterable[RunArg]) -> list[RunArg]:
         if self.root:
             return super().prepare_command(["sudo", *args])
         else:
@@ -220,7 +244,7 @@ class LocalDriver(SubprocessDriver):
 
     async def run_(
         self,
-        *args: Union[str, Path],
+        *args: RunArg,
         check: bool = True,
         input: Optional[bytes] = None,  # pylint:disable=redefined-builtin
         capture_output: bool = False,
@@ -228,9 +252,9 @@ class LocalDriver(SubprocessDriver):
     ) -> RunResult:
         if args and args[0] == "sudo":
             # If the sudo prompt is needed, avoid drawing a progress bar over it
-            # with first prompting for a no-op command.
+            # by authenticating the user first.
             async with TERMINAL_LOCK:
-                await super().run("sudo", "true")
+                await super().run("sudo", "-v")
         return await super().run_(
             *args,
             check=check,
