@@ -1,13 +1,13 @@
 import argparse
+from pathlib import Path
 from typing import Optional
 
 import trio
 from typed_argparse import TypedArgs
 
 from .driver import LocalDriver
-from .package import Package, load_packages
+from .manager import Manager
 from .state import DB, DB_PATH
-from .utils import flatten, parallel_map_tqdm
 
 
 class Args(TypedArgs):
@@ -21,26 +21,20 @@ def parse_args(args: Optional[list[str]] = None) -> Args:
 
 
 async def main():
+    args = parse_args()
+
     db = DB(DB_PATH)
     driver = LocalDriver()
-    args = parse_args()
+    manager = Manager(db=db, driver=driver, component_path=Path("packages"))
+
     components: frozenset[str] = frozenset(args.component) | {"base"}
-    packages = flatten(
-        load_packages(component, db=db, driver=driver) for component in components
-    )
 
-    async def process(package: Package) -> bool:
-        return await package.ensure()
+    installed = await manager.install(components)
 
-    async def process_and_record(package: Package) -> Optional[str]:
-        if await process(package):
-            return package.name
-        return None
-
-    results = await parallel_map_tqdm(process_and_record, packages)
-    installed = list(filter(None, results))
     if installed:
-        print(f"{len(installed)} packages installed or updated: {', '.join(installed)}")
+        print(
+            f"{len(installed)} packages installed or updated: {', '.join(p.name for p in installed)}"
+        )
     else:
         print("Everything up to date.")
 
