@@ -3,8 +3,10 @@ from abc import ABCMeta, abstractmethod
 from pathlib import Path
 from typing import Iterable, Union
 
+from ..configparser import DesktopEntry
 from ..utils import async_cached
 from .manual import ManualPackage
+from .tracked import Tracker
 
 
 class ArchivePackage(ManualPackage, metaclass=ABCMeta):
@@ -29,10 +31,9 @@ class ArchivePackage(ManualPackage, metaclass=ABCMeta):
     def pathname(self) -> str:
         return self.name.replace("/", "--")
 
+    @async_cached
     async def package_directory(self) -> Path:
-        result = (await self.local()) / "mybox" / self.pathname
-        await self.driver.makedirs(result)
-        return result
+        return (await self.local()) / "mybox" / self.pathname
 
     @async_cached
     async def tar(self) -> str:
@@ -117,9 +118,6 @@ class ArchivePackage(ManualPackage, metaclass=ABCMeta):
         )
 
     async def binary_path(self, binary: str) -> Path:
-        appimage_candidate = await self.package_directory() / "squashfs-root" / "AppRun"
-        if await self.driver.is_executable(appimage_candidate):
-            return appimage_candidate
         return await self.find_in_package_directory(
             paths=[[], ["bin"]],
             name=binary,
@@ -129,26 +127,15 @@ class ArchivePackage(ManualPackage, metaclass=ABCMeta):
 
     async def app_path(self, name: str) -> Path:
         return await self.find_in_package_directory(
-            paths=[["share", "applications"], ["squashfs-root"]],
+            paths=[["share", "applications"]],
             name=f"{name}.desktop",
             target_desc="application",
         )
 
     async def icon_paths(self, name: str) -> Iterable[Path]:
-        return [
-            Path(path)
-            for path in (
-                await self.driver.run_output(
-                    "find",
-                    await self.package_directory(),
-                    "-name",
-                    f"{name}.svg",
-                    "-o",
-                    "-name",
-                    f"{name}.png",
-                )
-            ).splitlines()
-        ]
+        return await self.driver.find(
+            await self.package_directory(), name=[f"{name}.svg", f"{name}.png"]
+        )
 
     async def font_path(self, name: str) -> Path:
         candidate = await self.package_directory() / name
@@ -158,7 +145,38 @@ class ArchivePackage(ManualPackage, metaclass=ABCMeta):
             f"Cannot find font '{name}' in {await self.package_directory()}."
         )
 
-    async def install(self):
+    async def install_appimage(self, tracker: Tracker) -> None:
+        app_dir = await self.package_directory() / "squashfs-root"
+        app_run = app_dir / "AppRun"
+        if not await self.driver.is_executable(app_run):
+            raise ValueError("AppImage does not have an executable named 'AppRun'.")
+
+        binary_name = self.pathname
+        binary_target = await self.local() / "bin" / binary_name
+        await self.install_binary_wrapper(app_run, binary_target)
+        tracker.track(binary_target)
+
+        desktop_files = await self.driver.find(app_dir, name="*.desktop", maxdepth=1)
+        if not desktop_files:
+            raise ValueError("AppImage does not have a .desktop file.")
+        if len(desktop_files) > 1:
+            raise ValueError(f"AppImage has multiple .desktop files: {desktop_files}")
+        desktop_file = desktop_files[0]
+
+        desktop_entry = DesktopEntry.from_string(
+            await self.driver.read_file(desktop_file)
+        )
+        desktop_entry.exec = " ".join(
+            [binary_name, desktop_entry.exec.split(" ", 1)[1]]
+        )
+
+        target_desktop_file = await self.application_path() / desktop_file.name
+        await self.driver.write_file(target_desktop_file, desktop_entry.to_string())
+        tracker.track(target_desktop_file)
+        if desktop_entry.icon:
+            await self.install_icon(desktop_entry.icon, tracker)
+
+    async def install_tracked(self, *, tracker: Tracker):
         url = await self.archive_url()
         async with self.driver.tempfile() as archive_path:
             await self.driver.run(
@@ -170,5 +188,15 @@ class ArchivePackage(ManualPackage, metaclass=ABCMeta):
                 archive_path,
                 url,
             )
+
+            package_directory = await self.package_directory()
+            await self.driver.makedirs(package_directory)
+            tracker.track(package_directory.parent)
+            tracker.track(package_directory)
+
             await self.extract(url, archive_path)
-        await super().install()
+
+        if url.endswith(".AppImage"):
+            await self.install_appimage(tracker)
+
+        await super().install_tracked(tracker=tracker)
