@@ -5,9 +5,10 @@ from typing import Iterator
 from ..utils import Some, unsome_
 from .destination import Destination
 from .manual_version import ManualVersion
+from .tracked import Tracked, Tracker
 
 
-class Links(ManualVersion, Destination):
+class Links(ManualVersion, Destination, Tracked):
     def __init__(
         self,
         links: str,
@@ -47,9 +48,9 @@ class Links(ManualVersion, Destination):
             m.update(str(path).encode())
         return m.hexdigest()
 
-    async def install(self) -> None:
-        # TODO: remove links that were created before but no longer exist
+    async def install_tracked(self, *, tracker: Tracker) -> None:
         destination = await self.destination()
+
         for path in self.paths():
             target = path.relative_to(self.source)
             if self.dot:
@@ -58,4 +59,6 @@ class Links(ManualVersion, Destination):
             target = destination.joinpath(target)
 
             await self.driver.link(path, target)
-        await super().install()
+            tracker.track(target, root=self.root)
+
+        await super().install_tracked(tracker=tracker)
