@@ -39,33 +39,43 @@ def unsome(x: Some[T]) -> list[T]:
     return unsome_(x) or []
 
 
+def intercalate(delimiter: T, items: Iterable[Iterable[T]]) -> list[T]:
+    it = iter(items)
+    try:
+        result = list(next(it))
+    except StopIteration:
+        return []
+    for x in it:
+        result.append(delimiter)
+        result.extend(list(x))
+    return result
+
+
 TERMINAL_LOCK = trio.Lock()
 
 
-async def run(*args: Union[str, Path]) -> subprocess.CompletedProcess:
+RunArg = Union[str, Path]
+
+
+async def run(*args: RunArg) -> subprocess.CompletedProcess:
     return await trio.run_process(args, check=True)
 
 
-async def run_ok(*args: Union[str, Path]) -> bool:
+async def run_ok(*args: RunArg) -> bool:
     try:
-        return (
-            await trio.run_process(
-                args,
-                check=False,
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-            )
-        ).returncode == 0
+        result = await trio.run_process(
+            args, check=False, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
+        )
+        return result.returncode == 0
     except FileNotFoundError:
         return False
 
 
-async def run_output(*args: Union[str, Path]) -> str:
-    return (
-        (await trio.run_process(args, capture_stdout=True, check=True))
-        .stdout.decode()
-        .strip()
+async def run_output(*args: RunArg, silent: bool = False) -> str:
+    result = await trio.run_process(
+        args, capture_stdout=True, capture_stderr=silent, check=True
     )
+    return result.stdout.decode().strip()
 
 
 def flatten(items: Iterable[Iterable[T]]) -> list[T]:
@@ -157,57 +167,46 @@ def raise_(exception: BaseException) -> Any:
     raise exception
 
 
+AsyncRet = Coroutine[Any, Any, T]
+
+
 @overload
-def async_cached(
-    fn: Callable[[], Coroutine[Any, Any, T]]
-) -> Callable[[], Coroutine[Any, Any, T]]:
+def async_cached(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]:
     ...
 
 
 @overload
-def async_cached(
-    fn: Callable[[U], Coroutine[Any, Any, T]]
-) -> Callable[[U], Coroutine[Any, Any, T]]:
+def async_cached(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]:
     ...
 
 
 @overload
-def async_cached(
-    fn: Callable[[U, V], Coroutine[Any, Any, T]]
-) -> Callable[[U, V], Coroutine[Any, Any, T]]:
+def async_cached(fn: Callable[[U, V], AsyncRet[T]]) -> Callable[[U, V], AsyncRet[T]]:
     ...
 
 
-def async_cached(
-    fn: Callable[..., Coroutine[Any, Any, T]]
-) -> Callable[..., Coroutine[Any, Any, T]]:
+def async_cached(fn: Callable[..., AsyncRet[T]]) -> Callable[..., AsyncRet[T]]:
     return _async_cached_lock(None, fn)
 
 
 @overload
-def async_cached_lock(
-    fn: Callable[[], Coroutine[Any, Any, T]]
-) -> Callable[[], Coroutine[Any, Any, T]]:
+def async_cached_lock(fn: Callable[[], AsyncRet[T]]) -> Callable[[], AsyncRet[T]]:
+    ...
+
+
+@overload
+def async_cached_lock(fn: Callable[[U], AsyncRet[T]]) -> Callable[[U], AsyncRet[T]]:
     ...
 
 
 @overload
 def async_cached_lock(
-    fn: Callable[[U], Coroutine[Any, Any, T]]
-) -> Callable[[U], Coroutine[Any, Any, T]]:
+    fn: Callable[[U, V], AsyncRet[T]]
+) -> Callable[[U, V], AsyncRet[T]]:
     ...
 
 
-@overload
-def async_cached_lock(
-    fn: Callable[[U, V], Coroutine[Any, Any, T]]
-) -> Callable[[U, V], Coroutine[Any, Any, T]]:
-    ...
-
-
-def async_cached_lock(
-    fn: Callable[..., Coroutine[Any, Any, T]]
-) -> Callable[..., Coroutine[Any, Any, T]]:
+def async_cached_lock(fn: Callable[..., AsyncRet[T]]) -> Callable[..., AsyncRet[T]]:
     return _async_cached_lock(trio.Lock(), fn)
 
 
@@ -223,8 +222,8 @@ NO_LOCK = NoLock()
 
 
 def _async_cached_lock(
-    lock: Optional[trio.Lock], fn: Callable[..., Coroutine[Any, Any, T]]
-) -> Callable[..., Coroutine[Any, Any, T]]:
+    lock: Optional[trio.Lock], fn: Callable[..., AsyncRet[T]]
+) -> Callable[..., AsyncRet[T]]:
     cache: dict[Any, T] = {}
 
     @wraps(fn)
